;;********************************************************************************************************************************
;;* Title: Instituional Bundle Theory                                                                                            *
;;* Designed by: Alhaji Cherif and Romarie Morales                                                                               *
;;*                                                                                                                              *
;;* The following code simulate a dynamic institutional bundle theory originally developed by one of the current authors (AC)    *
;;*                                                                                                                              *
;;*                                                                                                                              *
;;*                                                                                                                              *
;;********************************************************************************************************************************

links-own [ ]; rewired?   ;; keeps track of whether the link has been rewired or not

turtles-own
[
    unstable?               ;; if true, the node is unstable
    collapsed?              ;; if true, we assume that the node collapses and can no longer become stable
    tick-timer              ;; number of ticks since this turtle's last tick
    node-failed-threshold   ;; the threshold value above which the node becomes unstable
    K-measure?              ;; K-measure is load factor or concentration or stress distribution factor 
]

;;Set up Procedure
to setup
    clear-all
    setup-nodes
    setup-spatially-clustered-network
    ask n-of initial-unstable-node turtles [ unstable ]
    ask links [ set color gray]
    update-plot
end

;; Node Setup procedure
to setup-nodes
    set-default-shape turtles "circle"
    ; Turtle Creation
    create-turtles number-of-nodes
    [
      setxy ( random-xcor * 0.9 ) ( random-ycor * 0.9 )
      setxy random-xcor random-ycor
      ;give each node a random failed threshold within a given average set by the user
      ;this threshold is the threshold of failure above which the node failed
      set node-failed-threshold random-exponential node-failed-threshold-mean; random-float 100;
      set K-measure? 1 
      stable
      set tick-timer random tick-frequency
    ]
end

;; Link creation procedure
to setup-spatially-clustered-network
    ;; Creates links between nodes
    let num-links ( average-node-degree * number-of-nodes ) / 2
    while [ count links < num-links ]
    [
      ask one-of turtles
      [
        let choice ( min-one-of ( other turtles with [ not link-neighbor? myself ] ) [ distance myself ] )
        if choice != nobody [ create-link-with choice ]
      ]
    ]
    ;; Makes Network Display look better 
    repeat 10 [ layout-spring turtles links 0.5 ( world-width / ( sqrt number-of-nodes ) ) 1 ]  
end

;;Go Procedure
to go
    if all? turtles [ not unstable? ] [ stop ]
    ask turtles
    [ 
       set tick-timer tick-timer + 1 
       if tick-timer >= tick-frequency [ set tick-timer 0 ]
    ]
    spread
    ticking
    tick
    update-plot
end

                ;;*****************************
                ;;* Node Attribute Procedure  *
                ;;*****************************

;; Node Instability Procedure
to unstable
      ;ask turtles
      ;[
        if (1 - exp ( node-failed-threshold / ( random-float node-failed-threshold-mean + 1)) ^ beta ) < random-float 100 
        [
          set unstable? true
          set color orange
          set collapsed? false
        ]
      ;]
end

;; Node Stability Procedure
to stable  ;; 
      set unstable? false
      set color green
      set collapsed? false
end

;; Node Collapse Procedure
to collapsed
    set unstable? false
    set collapsed? true
    set color red
    ask my-links [ set color red - 2 ]
end

;; Node Spread/Contagion Procedure
to spread
    ask turtles with [ unstable? ]
      [  
          ask link-neighbors with [ not collapsed? ]
          [ ;; include formula later
            ;if random-exponential beta < spread-chance [ unstable ] 
            if random-float 100 < spread-chance [ unstable ]
          ] 
      ]
end

;; Node tick (counting) Procedure
to ticking
    ask turtles with [unstable? and tick-timer = 0]
    [ 
      if random-float 100 < collapsed-chance ; 1 - exp (random-float 100 < collapsed-chance 
      [ 
        ifelse random-float 100 < perminant-collapse-chance ;  K-measure? * node-failed-threshold  < perminant-collapse-chance
          [ 
            collapsed 
          ]
          [ 
            stable 
            set node-failed-threshold node-failed-threshold + random-float (random node-failed-threshold-mean)
          ]
       ]
     ]
      ask turtles with [ not unstable? and not collapsed? ]
      [
        if (count link-neighbors with [ not unstable? and not collapsed?] > 0) 
        [
          set K-measure? 1 + ( ( count link-neighbors with [collapsed?] ) / ( count link-neighbors with [ not unstable? and not collapsed?] ) ); + K-measure?
        ]
      ]
end

;; Plotting Procedure
to update-plot
    set-current-plot "Network Status"
    set-current-plot-pen "stable"
    plot ( (count turtles with [not unstable? and not collapsed?]) / (count turtles) * 100 )
    set-current-plot-pen "unstable"
    plot ( (count turtles with [unstable?]) / (count turtles) * 100 )
    set-current-plot-pen "collapsed"
    plot ( (count turtles with [collapsed?]) / (count turtles) * 100 )
  
    set-current-plot "K-measure"
    if (count turtles with [not unstable? and not collapsed? ]) / (count turtles) * 100 > 0 
    [ 
      plot ( sum [K-measure?] of turtles with [ not unstable? and not collapsed? ]) / ( count turtles with [ not unstable? and not collapsed? ] ) 
    ]
    set-current-plot "kmeasure histo"
    histogram [K-measure?] of turtles
  
    set-current-plot "threshold"
    plot (sum [node-failed-threshold] of turtles  / count turtles )
    set-current-plot "node-failed-threshold histo"
    histogram [node-failed-threshold] of turtles
    
    set-current-plot "Log-scale Threshold"
    plot ( log (sum [node-failed-threshold] of turtles  / count turtles ) 2)
    
    set-current-plot "Log-scale K-measure"
    if (count turtles with [not unstable? and not collapsed?] > 0 )
    [ 
      plot ( log ( ( sum [K-measure?] of turtles with [ not unstable? and not collapsed? ]) / ( count turtles with [ not unstable? and not collapsed? ] ) ) 2 )
    ]
    set-current-plot "Network Status log-scale"
    set-current-plot-pen "stable"
    if (count turtles with [not unstable? and not collapsed? ]) > 0 [ plot ( log ( (count turtles with [not unstable? and not collapsed?]) / (count turtles) * 100 ) 2 ) ]
    set-current-plot-pen "unstable"
    if (count turtles with [unstable?]) > 0 [ plot ( log ( (count turtles with [unstable?]) / (count turtles) * 100 ) 2 ) ]
    set-current-plot-pen "collapsed"
    if (count turtles with [collapsed?]) > 0 [ plot ( log ( (count turtles with [collapsed?]) / (count turtles) * 100 ) 2 ) ]
    
    set-current-plot "node-failed-threshold log-scale histo"
    histogram [ log node-failed-threshold 2] of turtles
    
    set-current-plot "kmeasure log-scale histo"
    histogram [log K-measure? 2] of turtles
end
@#$#@#$#@
GRAPHICS-WINDOW
730
10
1103
404
20
20
8.854
1
10
1
1
1
0
0
0
1
-20
20
-20
20
1
1
1
ticks

CC-WINDOW
5
677
1414
772
Command Center
0

SLIDER
13
240
210
273
perminant-collapse-chance
perminant-collapse-chance
0.0
100
31
1
1
%
HORIZONTAL

SLIDER
13
203
208
236
collapsed-chance
collapsed-chance
0.0
10.0
1.1
0.1
1
%
HORIZONTAL

SLIDER
13
127
208
160
spread-chance
spread-chance
0.0
10.0
2.5
0.1
1
%
HORIZONTAL

BUTTON
16
360
111
400
NIL
setup
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL

BUTTON
113
361
210
401
NIL
go
T
1
T
OBSERVER
NIL
NIL
NIL
NIL

PLOT
12
409
369
661
Network Status
time
% of nodes
0.0
52.0
0.0
100.0
true
true
PENS
"stable" 1.0 0 -10899396 true
"unstable" 1.0 0 -2674135 true
"collapsed" 1.0 0 -955883 true

SLIDER
12
10
207
43
number-of-nodes
number-of-nodes
10
100
100
5
1
NIL
HORIZONTAL

SLIDER
13
165
208
198
tick-frequency
tick-frequency
1
20
1
1
1
ticks
HORIZONTAL

SLIDER
12
88
207
121
initial-unstable-node
initial-unstable-node
1
10
1
1
1
NIL
HORIZONTAL

SLIDER
12
48
207
81
average-node-degree
average-node-degree
1
50
10
1
1
NIL
HORIZONTAL

SLIDER
13
279
209
312
node-failed-threshold-mean
node-failed-threshold-mean
0
20
7
1
1
NIL
HORIZONTAL

SLIDER
15
319
210
352
beta
beta
0
5
1.5
.1
1
NIL
HORIZONTAL

PLOT
216
210
469
403
kmeasure histo
NIL
NIL
0.0
10.0
0.0
10.0
true
false
PENS
"kmeasure" 1.0 1 -16777216 true

PLOT
216
10
468
211
K-measure
NIL
NIL
0.0
10.0
0.0
10.0
true
true
PENS
"K-measure" 1.0 0 -16777216 true

PLOT
468
10
726
210
threshold
NIL
NIL
0.0
10.0
0.0
10.0
true
true
PENS
"threshold" 1.0 0 -16777216 true

PLOT
469
210
726
404
node-failed-threshold histo
NIL
NIL
0.0
10.0
0.0
10.0
true
false
PENS
"threshold" 1.0 1 -16777216 true

PLOT
368
409
732
662
Log-scale Threshold
NIL
NIL
0.0
10.0
0.0
10.0
true
true
PENS
"Threshold" 1.0 0 -16777216 true

PLOT
732
410
1105
663
Log-scale K-measure
NIL
NIL
0.0
10.0
0.0
10.0
true
true
PENS
"K-measure" 1.0 0 -16777216 true

PLOT
1108
10
1405
212
Network Status log-scale
NIL
NIL
0.0
10.0
0.0
10.0
true
true
PENS
"stable" 1.0 0 -10899396 true
"unstable" 1.0 0 -2674135 true
"collapsed" 1.0 0 -955883 true

PLOT
1109
211
1405
406
node-failed-threshold log-scale histo
NIL
NIL
0.0
10.0
0.0
10.0
true
false
PENS
"default" 1.0 1 -16777216 true

PLOT
1109
410
1404
662
kmeasure log-scale histo
NIL
NIL
0.0
10.0
0.0
10.0
true
false
PENS
"default" 1.0 1 -16777216 true

@#$#@#$#@
WHAT IS IT?
-----------
This model demonstrates the spread of collapse through a network.  The model is abstract but has many applications in various fields.  One such application is based on the interpretation that each node represents a firm, and we are modeling the progress of collapse of a firm and its partners represented by link in this network.  Each node may be in one of three states:  stable, unstable, or collapsed.  This model was originally developed by one of the authors (AC) name below, and the model is called Institutional Bundle Theory.

----------------------
To refer to this model in academic publications, please use:  Cherif, A. and Morales, R. (2008).  NetLogo Instutional Bundle Theory. Arizona State University, Tempe, AZ.

To cite the original work in academic publication, please use: Cherif, A. Sociopolitical Bundle and Reliability theories under Nonlinear Political Interactions: Sociopolitical Instabilities and Political Dynamics of African States from 1946-2004. Cornell University, Ithaca, NY (Undergraduate Thesis)
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 4.0.3
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 0.0 1.0
0.0 1 1.0 0.0
0.2 0 0.0 1.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180

@#$#@#$#@
